// Audio FFT (v1.1.0)

var AudioFFT_use_DSP// = true

var AudioFFT = {
  SAMPLE_SIZE: 2048
 ,SAMPLE_RATE: 44100
 ,_fft: []
 ,spectrum: null

 ,use_live_input: returnBoolean("UseAudioFFTLiveInput")

 ,initialized: false
 ,init: function () {
if (this.initialized)
  return
this.initialized = true

DEBUG_show("Use audio FFT" + ((this.use_live_input) ? " (LIVE)" : ""), 2)

if (is_SA_child_animation)
  parent.AudioFFT_active = this
else
  self.AudioFFT_active = this

var bin_factor = (xul_mode) ? 0.45 : 1

//http://en.wikipedia.org/wiki/Decibel#Definition
//http://stackoverflow.com/questions/14169317/interpreting-web-audio-api-fft-results
var dB_gain = (this.use_live_input) ? (Math.log(Settings.UseAudioFFTLiveInputGain) / Math.LN10) * 10 / 0.5 : 0
if (bin_factor < 1)
  dB_gain /= Math.pow(bin_factor, 2/10)

//http://documentation.apple.com/en/soundtrackpro/usermanual/index.html#chapter=B%26section=1%26tasks=true
// 20-20k Hz, mid = 1k, exponential graph (when x=0, it's mid 1k(y=1))
var e_max = Math.log(20*bin_factor)

this.bin_factor = bin_factor
this.dB_gain = dB_gain
this.e_max = e_max

var EQbands = 16
BeatDetektor.config_default.BD_DETECTION_RANGES = EQbands
this.bd = new BeatDetektor();
this.kick_det = new BeatDetektor.modules.vis.BassKick();
this.vu = new BeatDetektor.modules.vis.VU();

var audio_context = (self.AudioContext || self.webkitAudioContext || self.mozAudioContext);
this.context = new audio_context();

if (AudioFFT_use_DSP) {
  this.proc = this.context.createScriptProcessor(this.SAMPLE_SIZE/2,1,1);
  this.proc.onaudioprocess = function (e) { AudioFFT.process_DSP(e); }

  this.signal = new Float32Array(this.SAMPLE_SIZE/2);

  Object.defineProperty(this, "fft",
{
  get: function () {
if (!this._fft[this.SAMPLE_RATE]) {
  this._fft[this.SAMPLE_RATE] = new FFT(this.SAMPLE_SIZE/2, this.SAMPLE_RATE)
  this.spectrum = []
}
return this._fft[this.SAMPLE_RATE]
  }
});
}
else {
  this.analyser = this.context.createAnalyser();
  this.analyser.smoothingTimeConstant = 0
  this.analyser.fftSize = 512*2;
}
  }

 ,initialized_live: false
 ,init_live: function () {
if (this.initialized_live)
  return
this.initialized_live = true

navigator.getMedia = ( navigator.getUserMedia ||
                       navigator.webkitGetUserMedia ||
                       navigator.mozGetUserMedia ||
                       navigator.msGetUserMedia);

navigator.getMedia({video: false, audio: true}, function(stream) {
//        console.log("mic : " + stream.audioTracks[0].label + '/' + stream.audioTracks.length);
  AudioFFT.live_stream = stream
  AudioFFT.connect()
},
function(e){
  DEBUG_show("getUserMedia ERROR:" + e,0,1)
});
  }

 ,connect: function (media_obj) {
if (media_obj) {
  var ao = (is_SA_child_animation) ? parent.AudioFFT_active : self.AudioFFT_active
  if (ao && ao.use_live_input)
    return

  if (this.media_obj == media_obj)
    return
  this.media_obj = media_obj
}

this.init()

if (this.live_stream) {
  if (this.source)
    return

  var source = this.context.createMediaStreamSource(this.live_stream);
  var gain = this.context.createGain()
//  gain.gain.value = Settings.UseAudioFFTLiveInputGain
  source.connect(gain)

  this.source = gain
}
else {
  var source_new = this.context.createMediaElementSource(media_obj);
  if (this.source) {
    if (AudioFFT_use_DSP) {
      this.source.disconnect();
      this.proc.disconnect();
    }
    else {
      this.source.disconnect();
      this.analyser.disconnect();
    }
  }
  this.source = source_new;
}

if (AudioFFT_use_DSP) {
  this.source.connect( this.proc );
  this.proc.connect( this.context.destination );

  if (!this.live_stream)
    this.source.connect( this.context.destination );
}
else {
  this.source.connect(this.analyser);

  if (!this.live_stream)
    this.analyser.connect(this.context.destination);
}
  }

 ,process_DSP: function (e) {
var m = this.media_obj
if (m) {
  if (m.paused || m.ended || !m.currentTime) {
    this.rEQBand = this.rBD = null
    if (m.ended || !m.currentTime)
      this._rEQBand = this._rBD = null
    return
  }
}

// FFT START
var
  buffers = [],
  channels = e.inputBuffer.numberOfChannels,
  i;

this.SAMPLE_RATE = e.inputBuffer.sampleRate
//DEBUG_show(e.inputBuffer.sampleRate)

for ( i = channels; i--; ) {
  buffers.push( e.inputBuffer.getChannelData( i ) );
}

if (channels > 1) {
  var resolution = this.SAMPLE_SIZE / channels;
  var sum = function ( prev, curr ) { return prev[ i ] + curr[ i ]; }
  for ( i = 0; i < resolution; i++ ) {
    this.signal[ i ] = buffers.reduce( sum ) / channels;
  }

  this.fft.forward(this.signal);
}
else {
  this.fft.forward(buffers[0]);
}
//END

this.spectrum.push(this.fft.spectrum)
//EV_sync_update.fps_count_func()
  }

 ,bin_table: []
 ,process: function () {
var m = this.media_obj
if (m) {
  if (m.paused || m.ended || !m.currentTime) {
    this.rEQBand = this.rBD = null
    if (m.ended || !m.currentTime) {
      var rHost = (is_SA_child_animation) ? parent : self
      rHost._rEQBand = rHost._rBD = null
    }
    return
  }
}
else {
  this.init_live()
  if (!this.source)
    return
}

var BD_mod
var fft, fft16, fft32
fft = fft16 = []

var divider_list = (Settings.Use32BandSpectrum) ? [16,32] : [16]

if (AudioFFT_use_DSP) {
  var spectrum = AudioFFT.spectrum
  if (!spectrum)
    return

  var length = spectrum.length
  if (!length)
    return

  var bin_size = this.SAMPLE_SIZE/4/16 * 3/4

  if (length == 1) {
for (var i = 0; i < 16; i++) {
  var v = 0
  var ii = i * bin_size
  for (var b = 0; b < bin_size; b++)
    v += spectrum[0][ii + b]
  fft[i] = v / bin_size
}
  }
  else if (length == 2) {
for (var i = 0; i < 16; i++) {
  var v = 0
  var ii = i * bin_size
  for (var b = 0; b < bin_size; b++)
    v += Math.max(spectrum[0][ii + b], spectrum[1][ii + b])
  fft[i] = v / bin_size
}
  }
  else {
for (var i = 0; i < 16; i++) {
  var v = 0
  var ii = i * bin_size
  for (var b = 0; b < bin_size; b++)
    v += Math.max(spectrum[0][ii + b], spectrum[1][ii + b], spectrum[2][ii + b])
  fft[i] = v / bin_size
}
  }

  for (var i = 0; i < 16; i++)
    fft[i] = parseInt(fft[i] * 10000 * (0.75 + 0.5 * i/15))
//DEBUG_show(fft)
  AudioFFT.spectrum = []

  BD_mod = 150
}
else {
  var length = this.analyser.frequencyBinCount
//  var freqData = new Uint8Array(length);
//  this.analyser.getByteFrequencyData(freqData);
  var freqData = new Float32Array(length);
  this.analyser.getFloatFrequencyData(freqData);

  var bin_factor = this.bin_factor
  var dB_gain = this.dB_gain
  var e_max = this.e_max

  length = parseInt(length * bin_factor)
  // add 0.0001 at the end to prevent accessing bins beyond the end due to round-up issue
  var bin_mid = length/(20*bin_factor) - 0.0001

  var powerData = []
//var peak_dB=0
  for (var i = 0; i < length; i++) {
    var dB = freqData[i] + dB_gain + 100
//if (peak_dB < dB) peak_dB = dB
    powerData[i] = Math.pow(10, (dB) / 10)
  }
//DEBUG_show(peak_dB)

for (var _d = 0, _d_max = divider_list.length; _d < _d_max; _d++) {
  var divider = divider_list[_d]
  var _fft = []

  var b_tag = bin_mid + "x" + divider
  var b_table = this.bin_table[b_tag]
  if (!b_table) {
//DEBUG_show(b_tag,0,1)
    b_table = this.bin_table[b_tag] = []

    var d1 = divider/2-1
    var d2 = divider/2
    var b_end = 0
    var power_factor = 2/3
    if (bin_factor < 1)
      power_factor += (1-power_factor) * (1-bin_factor)
    for (var i = 0; i < divider; i++) {
      var bb
      bb = b_table[i] = {}

      var f = Math.pow(Math.E, (i-d1)/d2 * e_max)
      bb.power_factor = Math.pow(f, power_factor)
      bb.ini = b_end
      bb.end = b_end = f * bin_mid
    }
  }

  var b_ini, b_end, b_ini_i, b_end_i
  var b_counter = 0
  for (var i = 0; i < divider; i++) {
    var bb = b_table[i]
    b_ini = bb.ini
    b_end = bb.end
    b_ini_i = parseInt(b_ini)
    b_end_i = parseInt(b_end)

    var power_total = 0
    var run_flag = true
    while (run_flag) {
      var scale = 1
      var counter_inc = false
      if (b_counter == b_ini_i)
        scale -= b_ini - b_ini_i
      if (b_counter == b_end_i) {
        run_flag = false
        scale -= (b_end_i+1) - b_end
        if (!scale)
          break
      }
      else
        counter_inc = true

      power_total += powerData[b_counter] * scale

      if (counter_inc)
        b_counter++
    }

    power_total *= bb.power_factor

    var dB = (Math.log(power_total) / Math.LN10) * 10
    dB = (dB < 30) ? 0 : (dB - 30) / 0.7
    dB = Math.pow(dB, 3)/2000

    _fft[i] = parseInt(dB)
  }

  if (_d == 0)
    fft = _fft
  self["fft" + divider] = _fft
}

  BD_mod = 500
}

// Beat detection START
var EQbands = 16

var EQ_for_bd = []
for (var i = 0; i < EQbands; i++) {
  EQ_for_bd[i] = fft[i] / BD_mod
}

this.bd.process((Date.now())/1000, EQ_for_bd);
this.kick_det.process(this.bd);
this.vu.process(this.bd);

var vu_levels = (this.vu.vu_levels.length) ? this.vu.vu_levels.slice(0,EQbands) : [0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0];
for (var i = 0; i < EQbands; i++)
  vu_levels[i] = parseInt(vu_levels[i] * 100 * 10/7)
/*
if (!this._vu_max) this._vu_max = 0
for (var i = 0; i < 16; i++) { if (this._vu_max < vu_levels[i]) this._vu_max = vu_levels[i] }
DEBUG_show(this._vu_max)
*/
var bass_kicked = this.kick_det.isKick()
// END

var rHost = (is_SA_child_animation) ? parent : self
this.rEQBand = rHost._rEQBand = '[' + fft.join(',') + ']'
this.rBD = rHost._rBD = '{"bass_kicked":' + bass_kicked + ',"vu_levels":[' + vu_levels.join(',') + ']}'

for (var _d = 1, _d_max = divider_list.length; _d < _d_max; _d++) {
  var divider = divider_list[_d]
  this["rEQBand" + divider] = rHost["_rEQBand" + divider] = '[' + self["fft" + divider].join(',') + ']'
}
  }
};


(function () {
  if (AudioFFT.use_live_input) {
    if (is_SA_child_animation && !parent.AudioFFT_active)
      parent.AudioFFT_active = AudioFFT
    else
      self.AudioFFT_active = AudioFFT
  }
})();


// beat detection
var AT_bass_band = 0
document.write('<script language="JavaScript" src="js/beatdetektor.js"></scr'+'ipt>')

if (AudioFFT_use_DSP)
  document.write('<script language="JavaScript" src="js/dsp.js"></scr'+'ipt>')
