// CANVAS Matrix Rain Effect v1.3.0

function random(num) {
  return Math.floor(Math.random() * num)
}

function MatrixRain(width,height, para) {
  this.width  = width
  this.height = height

  if (para) {
    for (var p in para)
      this[p] = para[p]
  }

  if (!this.grid_size)
    this.grid_size = 8//((width * height > 640 * 480) ? 12 : 8)
  if (!this.font)
    this.font = this.grid_size + "px Arial"

  if (this.mask_divider == null)
    this.mask_divider = 0
  if (this.mask_always_redraw == null)
    this.mask_always_redraw = true
  if (this.full_color == null)
    this.full_color = true

  this.rain_canvas = []
  this.rain = []
  this.rain_speed = []
  this.rain_head = []
  this.rain_tail = []

  this.canvas = document.createElement("canvas")
  this.canvas_mask = document.createElement("canvas")

  //misc, System Animator specific
  this._SA_active_count = 0
  this._SA_active_delay = 10
  this._SA_idle_count = this._SA_idle_count_max = 30
}

MatrixRain.prototype.char_list = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'

MatrixRain.prototype.matrixCreate = function (w,h) {
  if (!w)
    w = this.width
  if (!h)
    h = this.height
  this.width  = w
  this.height = h

  var canvas = this.canvas_matrix = document.createElement("canvas")
  canvas.width  = w
  canvas.height = h

  if (this.full_color) {
    this.canvas_matrix_copy = document.createElement("canvas")
    this.canvas_matrix_copy.width  = w
    this.canvas_matrix_copy.height = h
  }
  else {
    var context = canvas.getContext("2d")
    context.fillStyle = "#004000"
    context.fillRect(0,0,w,h)
  }

  var grid_size = this.grid_size
  var mw = this.matrix_width  = Math.ceil(w/this.grid_size)
  var mh = this.matrix_height = Math.ceil(h/this.grid_size)
  var char_list = this.char_list
  var char_list_max = char_list.length

  for (var x = 0; x < mw; x++) {
    var c = document.createElement("canvas")
    c.width  = grid_size
    c.height = mh * grid_size

    var context = c.getContext("2d")
    context.font = this.font

    var y_tail = parseInt(mh/5)
    for (var y = mh-1; y > 0; y--) {
      context.strokeStyle = (y == mh-1) ? "#FFFFFF" : "#00FF00"
      context.globalAlpha = (y >= y_tail) ? 1 : y / y_tail

      var py = (y+0.8) * grid_size
      context.strokeText(char_list.charAt(random(char_list_max)), 0,py)
    }

    this.rain_canvas[x] = c
  }

  this.matrixReset()
}

MatrixRain.prototype.matrixReset = function () {
  // will become 'true' when all rains have fallen completely
  this.matrix_entered = false

  // will become 'true' when rains begin fading away
  this.matrix_leaving = false

  var mw = this.matrix_width
  var mh = this.matrix_height
  for (var x = 0; x < mw; x++) {
    var y1 = -(random(mh/5)+1)//random(mh) + mh - 1
    var y2 = y1 - mh - (random(mh/5)+1)
    this.rain[x] = [y1, y2]
    this.rain_head[x] = y1

    var speed = random(3)
    this.rain_speed[x] = speed+1//(speed) ? speed : 0.5
  }
}

MatrixRain.prototype.matrixDraw = function (skip) {
  var canvas = this.canvas_matrix
  var w = canvas.width
  var h = canvas.height

  var context = canvas.getContext("2d")
  context.globalCompositeOperation = 'copy'

  if (skip) {
    if (this.full_color) {
      context.globalAlpha = 1
      context.drawImage(this.canvas_matrix_copy, 0,0)
    }
    return
  }

  context.globalAlpha = 0.5
  context.drawImage(canvas, 0,0)
  context.globalAlpha = 1
  if (!this.full_color) {
    context.globalCompositeOperation = 'destination-over'
    context.fillStyle = "#004000"
    context.fillRect(0,0,w,h)
  }

/*
  if (this.full_color) {
    context.clearRect(0,0,w,h)
  }
  else {
    context.fillStyle = "#004000"
    context.fillRect(0,0,w,h)
  }
*/
  context.globalCompositeOperation = 'source-over'

  var grid_size = this.grid_size
  var mw = this.matrix_width
  var mh = this.matrix_height
  var mhh = mh * grid_size

  var rain_still_falling = 0
  var rain_gone = 0

  for (var x = 0; x < mw; x++) {
    var rain = this.rain[x]
    var speed = this.rain_speed[x]
    var head = this.rain_head
    var tail = this.rain_tail

    rain[0] += speed
    rain[1] += speed

    var rain_c = this.rain_canvas[x]
    for (var i = 0; i < 2; i++) {
      var i_opposite = (i) ? 0 : 1
      if (parseInt(rain[i]) >= mh*2-1)
        rain[i] = (this.matrix_leaving) ? -9999 : rain[i_opposite] - mh - (random(mh/5)+1)

      var y = parseInt(rain[i])
      if (this.matrix_leaving) {
        if (y < -999)
          rain_gone++
        if (rain[i_opposite] < -999) {
          if (y < -999)
            tail[x] = mh-1
          else {
            var r = tail[x]
            if (!r)
              r = 0
            if (r < y-(mh-1))
              tail[x] = y-(mh-1)
          }
        }
      }
      else if (!this.matrix_entered) {
        if (head[x] < y)
          head[x] = y
        if (head[x] < mh)
          rain_still_falling++
      }

      if ((y >= mh*2-3) || (y < 1))
        continue

      var sy

      var px = x * grid_size
      var py = (y - (mh-2)) * grid_size

      var pw = (px + grid_size > w) ? w - px : grid_size

      var ph = (mh-1) * grid_size
      if (py < 0) {
        ph += py
        py = 0
      }
      if (py + ph > h)
        ph = h - py

      var sy = (y <= mh-1) ? mhh - ph : 0

      context.drawImage(rain_c, 0,sy,pw,ph, px,py,pw,ph)
    }
  }

  if (this.full_color) {
    context = this.canvas_matrix_copy.getContext("2d")
    context.globalCompositeOperation = 'copy'
    context.globalAlpha = 1
    context.drawImage(canvas, 0,0)
  }

  if (this.matrix_leaving) {
    if (rain_gone == mw*2)
      this.raining = false
  }
  else if (!rain_still_falling)
    this.matrix_entered = true
}

MatrixRain.prototype.maskDraw = function (obj, width,height) {
  var mask = this.canvas_mask
  var w = (width)  ? width  : obj.width
  var h = (height) ? height : obj.height

  var wxh = w + 'x' + h
  if (!this.mask_always_redraw && (mask.drawn == wxh))
    return
  mask.drawn = wxh

  var divider = this.mask_divider
  if (!divider) {
    if (this.mask_always_redraw) {
      divider = Math.sqrt(w*h) / 250
      if (divider < 1)
        divider = 1
    }
    else
      divider = 1
  }

  var dw,dh
  if (divider == 1) {
    dw = w
    dh = h
  }
  else {
    dw = parseInt(w/divider)
    dh = parseInt(h/divider)
//DEBUG_show(dw+'x'+dh)
  }
  mask.width  = dw
  mask.height = dh

  var context = mask.getContext("2d")
  context.globalCompositeOperation = 'copy'
  context.drawImage(obj, 0,0,dw,dh)

  var frame = context.getImageData(0,0,dw,dh)
  var data = frame.data

  for (var i = 0, data_length = data.length; i < data_length; i+=4) {
//http://en.wikipedia.org/wiki/Grayscale#Converting_color_to_grayscale
//(0.3* color.R + 0.59 * color.G + 0.11 * color.B);
    var l = parseInt(0.3*data[i] + 0.59*data[i+1] + 0.11*data[i+2])
    if (l < 250)
      data[i+3] = l
  }

  context.putImageData(frame, 0,0)
}

MatrixRain.prototype.draw = function (obj, width,height) {
  var canvas = this.canvas
  var w = (width)  ? width  : obj.width
  var h = (height) ? height : obj.height

  if ((w > this.width) || (h > this.height)) {
    this.matrixCreate(w,h)
  }

  canvas.width  = w
  canvas.height = h

  var context = canvas.getContext("2d")
  if (this.full_color) {
    var m_canvas = this.canvas_matrix
    var m_context = m_canvas.getContext("2d")

    context.globalCompositeOperation = 'copy'
    context.drawImage(obj, 0,0,w,h)

/*
context.globalCompositeOperation = 'source-over'
context.fillStyle = "rgba(255,255,255, 0.5)"
var grid_size = this.grid_size
var mw = this.matrix_width
for (var x = 0; x < mw; x++) {
  var rain = this.rain[x]
  for (var i = 0; i < 2; i++) {
    var sy = parseInt(rain[i]) * grid_size
    if ((sy < 0) || (sy > h))
      continue

    var sx = x * grid_size
    var sw = (sx + grid_size > w) ? w - sx : grid_size
    var sh = (sy + grid_size > h) ? h - sy : grid_size

    context.fillRect(sx,sy,sw,sh)
  }
}
*/

    m_context.globalCompositeOperation = 'source-in'
    m_context.drawImage(canvas, 0,0)

/*
context.globalCompositeOperation = 'copy'
context.drawImage(obj, 0,0,w,h)
*/

    context.globalCompositeOperation = 'source-atop'
    context.fillStyle = "rgba(0,0,0, 0.75)"
    context.fillRect(0,0,w,h)

    context.globalCompositeOperation = 'source-over'
    context.drawImage(m_canvas, 0,0)
  }
  else {
    this.maskDraw(obj, w,h)

    context.globalCompositeOperation = 'copy'
    context.drawImage(this.canvas_matrix, 0,0)

    context.globalCompositeOperation = 'destination-in'
    context.drawImage(this.canvas_mask, 0,0,w,h)

    context.globalCompositeOperation = 'destination-atop'
    context.fillStyle = "#000000"
    context.fillRect(0,0,w,h)

    context.globalCompositeOperation = 'destination-in'
    context.drawImage(obj, 0,0,w,h)
  }

  if (this.content_mask) {
    context.globalCompositeOperation = 'destination-in'
    context.drawImage(this.content_mask_load(w,h), 0,0)

    context.globalCompositeOperation = 'destination-over'
    context.drawImage(obj, 0,0,w,h)
  }

  if (this.matrix_entered)
    return

  var grid_size = this.grid_size
  var rain = (this.matrix_leaving) ? this.rain_tail : this.rain_head

  context.save()

  context.beginPath()

  var px,py
  py = rain[0]
  if (!py)
    py = 0
  py *= grid_size
  if (py > h)
    py = h
  var py0 = py
  context.moveTo(0,py0)
  context.lineTo(grid_size,py0)

  for (var x = 1, max = this.matrix_width; x < max; x++) {
    py = rain[x]
    if (!py)
      py = 0
    py *= grid_size
    if (py > h)
      py = h
    px = x * grid_size
    context.lineTo(px,py)

    px += grid_size
    if (px > w)
      px = w
    context.lineTo(px,py)
  }

  var h_edge = (this.matrix_leaving) ? 0 : h
  context.lineTo(w,h_edge)
  context.lineTo(0,h_edge)
  context.lineTo(0,py0)

  context.clip()

  context.globalCompositeOperation = 'source-over'
  context.drawImage(obj, 0,0,w,h)

  context.restore()
}

Object.defineProperty(MatrixRain.prototype, "content_mask",
{
  get: function () {
return ((this.content_mask_obj) ? this.content_mask_ : "")
  }

 ,set: function (v) {
this.content_mask_ = v
this.content_mask_obj = null

var img = new Image()
img.matrix_rain = this
img.onload = function () {
  var matrix_rain = this.matrix_rain
  matrix_rain.content_mask_obj = this

  var c = matrix_rain.content_mask_canvas
  if (!c)
    c = matrix_rain.content_mask_canvas = document.createElement("canvas")
  c.width = c.height = 0
}

img.src = toFileProtocol(v)
  }
});

MatrixRain.prototype.content_mask_load = function (w,h) {
  var c = this.content_mask_canvas
  if ((c.width == w) && (c.height == h))
    return c

  c.width  = w
  c.height = h

  var context = c.getContext("2d")
  context.globalCompositeOperation = 'copy'

  var obj = this.content_mask_obj
  context.drawImage(obj, 0,0,obj.width,obj.height, 0,0,w,h)

  return c
}


// System Animator specific

var MatrixRain_para

Object.defineProperty(MatrixRain.prototype, "_SA_active",
{
  get: function () {
var count = (!this._PC_count_absolute) ? 1 : PC_count_absolute - this._PC_count_absolute
this._PC_count_absolute = PC_count_absolute

var idle = (this._SA_reversed) ? (EV_usage_float) : (!EV_usage_float)
if (idle) {
  this._SA_active_count = 0

  if (!this._SA_idle_count_max)
    return true
  if (!this.raining)
    return false

  this._SA_idle_count += count
  if (this._SA_idle_count >= this._SA_idle_count_max) {
    if (!this.matrix_leaving) {
      this.matrix_leaving = true
      this.matrix_entered = false
      this.rain_tail = []
    }
  }
}
else {
  this._SA_idle_count = 0

  this._SA_active_count += count
  if (this._SA_active_count == this._SA_active_delay) {
    this.raining = true
    if (this.matrix_leaving) {
      this.matrix_leaving = false
      this.matrixReset()
    }
  }
}

return this.raining
  }
});
